#include "CHierNav.H"
#include "cluster.H"
#include "ARR.H"
#include "ogl.H"
#include "colormap.H"
#include "XmdvTool.H"
#include "brush_oper.H"
#include <math.h>
#include <stdlib.h>

/* SID */
#include "db_buffer.h"

#define CONTOUR_RES 25

static ARR<double> contour_radius;
static ARR< ARR<double> > tree_contour;
static ARR< ARR<double> > tree_color;
static int max_display_height;

CHierNav::CHierNav(CHierDisplayInformation* hier_info) : CHierDisplay(hier_info)
{
//    cfNavInit();
}


static void FindTreeHeight(Cluster* node, int curr_ht, int *max_ht)
{
    if (curr_ht>*max_ht) *max_ht = curr_ht;

    for (int i=0; i<node->num_children; i++)
        FindTreeHeight(&btree->nodes[node->child[i]], curr_ht+1, max_ht);
}

static void FindRadiusDistribution()
{
    assert(histo.Len()!=0);

    int MIN_GAP = 5;

    double R = btree->nodes[0].sqrt_radius;

    contour_radius.SetLen(0);

    for (int i=histo.Len()-1, previ=1000000;i>=0;i--)
    {
        if (histo[i]==0) continue;
        if ((previ-i)<MIN_GAP) continue;

        contour_radius.InsertTail( R*i/double(histo.Len()-1) );
        previ = i;
    }
    contour_radius.InsertTail(0.0);
}

static void FindTreeContour(Cluster *node,int curr_ht,double rad,ARR<double> &contour,ARR<double> &color,double L,double R)
{
    if (node->parent != -1)
    {
        Cluster *parent = &btree->nodes[node->parent];

        //
        // If it's a clustered node and a leaf
        //
        if (node->num_children==0 && node->sqrt_radius >= rad) {
            rad = node->sqrt_radius+ 0.01; // (1.0/(double)CONTOUR_RES);
        }

        if (node->sqrt_radius < rad)
        {
            //
            // Iso-radius contour cuts between node and parent.
            // "interp" is the intersection ratio between node and parent.
            // "y" is the intersection height of the contour.
            //

            double interp;

            if (parent->sqrt_radius - node->sqrt_radius < 0.0000001) interp = 0;
            else
                interp = (rad - node->sqrt_radius) /
                           (parent->sqrt_radius - node->sqrt_radius);

            double y = curr_ht - interp;

            //
            // Find the contour segment(s) this node covers: [x0,x1]
            //
            double x0 = L*CONTOUR_RES;
            double x1 = R*CONTOUR_RES;

            int xi0 = (int) ceil(x0+.000000001);
            int xi1 = (int) floor(x1-.00000001);

            for (int i=xi0;i<xi1;i++) {
                // Coverage that fills whole segments
                assert(contour[i] == 0.0);
                contour[i] = y;
                color[i] = (L+R)*.5;
            }

            if (xi0>0 && x1>xi0) {
                // Left end of the coverage
                double coverage = (xi0-x0);
                contour[xi0-1] += coverage*y;
                if (coverage>.9) color[xi0-1] = (L+R)*.5;
            }
            if (xi1<CONTOUR_RES && x0<xi1) {
                // Right end of the coverage
                double coverage = (x1-xi1);
                contour[xi1] += coverage*y;
                if (coverage>.9) color[xi1] = (L+R)*.5;
            }
            if (xi0>=x1 && xi1<=x0) {
                // Partial coverage
                double coverage = (x1-x0);
                contour[xi1] += coverage*y;
                if (coverage>.9) color[xi1] = (L+R)*.5;
            }

            return;
        }
    }

    double k = (R-L)/(double)(node->num_children);

    for (int i=0; i<node->num_children; i++) {
        FindTreeContour(&btree->nodes[node->child[i]], curr_ht+1, rad, contour, color, L+i*k, L+(i+1)*k);
    }
}

void SetupBrushParameters()
{
    if ( !btree )
        return;
//    EXTENT_BRUSH = 0;
    hierInitExtents();
    hierClearNodeFlags(IS_BRUSHED);
    hierClearNodeFlags(IS_BRUSHED_ANY);
    hierMarkBrushedClustersByColor(&btree->nodes[0], hier_info->tree_brush[0], hier_info->tree_brush[1]);

//Mask out since data brush and sbb is treated as separate entities
//    hierModifyBrushParametersByExtents();
//    BrushOperMarkDataset();
}

void cfNavInit()
{
    FindRadiusDistribution();

    int i;

    hier_info->tree_height = 0;

    // Clear contour array
    //
    for (i=0;i<tree_contour.Len();i++) {
        tree_contour[i].Clear();
    }
    tree_contour.Clear();


    // Compute the contours using the average radii
    //
    tree_contour.SetLen(contour_radius.Len());
    tree_color.SetLen(contour_radius.Len());

    for (i=0;i<tree_contour.Len();i++) {
        tree_contour[i].SetLen(CONTOUR_RES);
        tree_color[i].SetLen(CONTOUR_RES);
        for (int j=0;j<CONTOUR_RES;j++) {
            tree_contour[i][j] = 0.0;
            tree_color[i][j] = -1.0;
        }
    }

    for (i=0;i<tree_contour.Len();i++) {
        FindTreeContour(&btree->nodes[0],0,contour_radius[i],tree_contour[i],tree_color[i],0,1);
    }

    //
    // Find the maximum depth of the contours, for display
    // purposes. The maximum depth may not be the maximum
    // height of the tree, because the contours are approximated.
    //
    max_display_height = 0;
    for (int j=0;j<CONTOUR_RES;j++) {
        if (tree_contour[tree_contour.Len()-1][j] > max_display_height)
            max_display_height = (int) ceil(tree_contour[tree_contour.Len()-1][j]);
    }
}

static void drawContours(double radius, double L, double R, double ypa, double width=1.0)
{
    //
    // Find the two bounding contour radii
    //
    int i;
    for (i=0;i<contour_radius.Len();i++) {
        if (radius > contour_radius[i]) break;
    }
    if (i==0) i++;
    else if (i==contour_radius.Len()) i--;

    //
    // Interpolate between the two contours to
    // get the approximated contour for the current
    // radius. Show colormap along the contour.
    //

    glLineWidth((float) width);
    glBegin(GL_LINE_STRIP);

    /* SID -- another division by zero */
    double y_interp = 0;
    if (contour_radius[i-1]-contour_radius[i])
	y_interp = (radius - contour_radius[i]) /
				(contour_radius[i-1]-contour_radius[i]);

    double x_gap = 1.0 / (CONTOUR_RES-1);

    for (int j=0;j<CONTOUR_RES;j++)
    {
        double y_hi = tree_contour[i][j];
        double y_lo = tree_contour[i-1][j];

        // x_interp is normalized between [0,1]
        double x_interp = j*x_gap;

        // Limit contour drawing to between L and R
        if (x_interp<L) {
            if ((x_interp+x_gap)<L) continue;
            double k = (L-x_interp)/x_gap;

            if (j<CONTOUR_RES-1) {
                y_hi = (1-k)*(tree_contour[i][j]) + k*tree_contour[i][j+1];
                y_lo = (1-k)*(tree_contour[i-1][j]) + k*tree_contour[i-1][j+1];
                x_interp = L;
            }
        }
        else
        if (x_interp>R) {
            if ((x_interp-x_gap)>R) continue;
            double k = (x_interp-R)/x_gap;

            if (j>0) {
                y_hi = (1-k)*(tree_contour[i][j]) + k*tree_contour[i][j-1];
                y_lo = (1-k)*(tree_contour[i-1][j]) + k*tree_contour[i-1][j-1];
                x_interp = R;
            }
        }

        double y = (1-y_interp)*y_hi + (y_interp)*y_lo;

        // xx is normalized between [-1,1]
        double xx = 2*x_interp - 1;

        // Now, normalize xx to trianglular coordinates
        xx *= y/(double)max_display_height;

        if (tree_color[i][j] != -1) { x_interp = tree_color[i][j]; }
        unsigned long color = hier_info->CMAP[ int(x_interp*(CMAPSIZE-1)) ];
        glColor3ub((unsigned char)(color>>16)&0xff, (unsigned char)(color>>8)&0xff, (unsigned char)color&0xff);

        glVertex2f((float) ((xx + 1) / 2.0), (float) (y * ypa));
    }

    glEnd();
    glLineWidth(1.0);
}

#define BORDER 5
#define TOP_BORDER BORDER
#define BOT_BORDER 5*BORDER
#define BARHIGHT 0.05
#define YPARA (1-BARHIGHT)/max_display_height

void CHierNav::Draw()
{
    if (!dataset || !btree) return;

    canvas->MakeCurrent();

    PrepareCanvas();

    canvas->SetForeground(0xaaaaaa);
    canvas->SetBackground(0x000000);
    canvas->Clear();
    //
    // Draw the brush and brush handles
    //
    glColor3f((float) .5, (float) .1, (float) .3);
    glBegin(GL_POLYGON);
    glVertex2f(0.5, 0);
    glVertex2f((float) (hier_info->tree_brush[0]), (float) (1-BARHIGHT));
    glVertex2f((float) (hier_info->tree_brush[1]), (float) (1-BARHIGHT));
    glEnd();

//    glDisable(GL_POLYGON_STIPPLE);

    glColor3f((float) .8, (float) .8, (float) .8);
    glPointSize(5.0);
    glBegin(GL_POINTS);
    glVertex2f((float) (hier_info->tree_brush[0]), (float) (1-BARHIGHT));
    glVertex2f((float) (hier_info->tree_brush[1]), (float) (1-BARHIGHT));
    glEnd();
    glPointSize(1.0);


    //
    // Draw the outer triangle
    //
    glColor3f((float) .8, (float) .8, (float) 0);
    glBegin(GL_LINE_LOOP);
    glVertex2f(0.5, 0);
    glVertex2f((float) (+1), (float) (1-BARHIGHT));
    glVertex2f((float) (0), (float) (1-BARHIGHT));
    glEnd();

    //
    // Draw the contours
    //
    glColor3f((float) .8, (float) .8, (float) .8);
    int i;
    for (i=0;i<tree_contour.Len();i++) 
    {
        i=tree_contour.Len()-1; // only draw the last contour
        glBegin(GL_LINE_STRIP);
        for (int j=0;j<CONTOUR_RES;j++) {
            double y = tree_contour[i][j];
            double x = ( 2*j/(double)(CONTOUR_RES-1) - 1 ) * y/(double)max_display_height;
//          printf("Vertex x[%d] = %lf y=%lf \n", j,x,y);
            glVertex2f((float) (x/2.+0.5), (float) (y * YPARA));
        }
        glEnd();
    }

    //
    // Draw the current contour
    //
    double r = btree->nodes[0].sqrt_radius;
    
    if ( hier_info->database )
    {
        //SID: drawContours(r*cluster_radius,0.,tree_brush[0], 2.0);
        drawContours(r*hier_info->brushed_radius,hier_info->tree_brush[0], hier_info->tree_brush[1], YPARA, 3.0);
        //SID: drawContours(r*cluster_radius,tree_brush[1], 1.0, 2.0);
    }
    else
    {
        drawContours(r*hier_info->cluster_radius, 0.,hier_info->tree_brush[0], YPARA, 2.0);
        drawContours(r*hier_info->brushed_radius, hier_info->tree_brush[0], hier_info->tree_brush[1], YPARA, 3.0);
        drawContours(r*hier_info->cluster_radius, hier_info->tree_brush[1], 1.0, YPARA, 2.0);
    }

    //
    // Draw the labels: tree depth, current depth
    //
    if (hier_info->tree_height == 0) FindTreeHeight(&btree->nodes[0],0,&(hier_info->tree_height));
    char str[30];
    sprintf(str,"Tree Depth:%d", hier_info->tree_height);
    canvas->SetForeground(0xffffff);
    canvas->DrawString(0,0.05,str);

    //
    // Draw color bar
    //
    glShadeModel(GL_SMOOTH);
    glLineWidth(float(5.0*canvas->zoom.Y));
    glBegin(GL_LINE_STRIP);
    int cmap_size = CMAPSIZE;
    double step = 1/double(cmap_size-1);
    for (i=0;i<cmap_size;i++) 
    {
        unsigned long color = hier_info->CMAP[i];
        glColor3ub((unsigned char)(color>>16)&0xff,(unsigned char)(color>>8)&0xff,(unsigned char)color&0xff);
        double k = i*step;
        glVertex2f((float) k,1);
    }
    glEnd();
    glLineWidth(1.0);

}

/*
void CHierNav::Draw()
{
    if (!dataset || !btree) return;

    canvas->MakeCurrent();

    //
    // Setup window parameters.
    //
    int W,H;
    canvas->GetSize(&W,&H);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    glOrtho(0,+1,1+.001,0,-1,1);
    glMatrixMode(GL_MODELVIEW);
    
    canvas->SetForeground(0xaaaaaa);
    canvas->SetBackground(0x000000);
    canvas->Clear();


//    glViewport( BORDER, BORDER, W-2*BORDER, 2*BORDER);
//    hier_info->CMAP.DrawColorBar();

//    glViewport( BORDER, BOT_BORDER, W-2*BORDER, H-BOT_BORDER-TOP_BORDER);
    glViewport( 0, 0, W, H);


    //
    // Draw the brush and brush handles
    //
    glColor3f((float) .5, (float) .1, (float) .3);
    glBegin(GL_POLYGON);
    glVertex2f(0.5, 0);
    glVertex2f((float) (hier_info->tree_brush[0]), (float) max_display_height);
    glVertex2f((float) (hier_info->tree_brush[1]), (float) max_display_height);
    glEnd();

//    glDisable(GL_POLYGON_STIPPLE);

    glColor3f((float) .8, (float) .8, (float) .8);
    glPointSize(5.0);
    glBegin(GL_POINTS);
    glVertex2f((float) (hier_info->tree_brush[0]), (float) max_display_height);
    glVertex2f((float) (hier_info->tree_brush[1]), (float) max_display_height);
    glEnd();
    glPointSize(1.0);


    //
    // Draw the outer triangle
    //
    glColor3f((float) .8, (float) .8, (float) 0);
    glBegin(GL_LINE_LOOP);
    glVertex2f(0.5, 0);
    glVertex2f((float) (+1), (float) max_display_height);
    glVertex2f((float) (0), (float) max_display_height);
    glEnd();

    //
    // Draw the contours
    //
    glColor3f((float) .8, (float) .8, (float) .8);
    int i;
    for (i=0;i<tree_contour.Len();i++) 
    {
        i=tree_contour.Len()-1; // only draw the last contour
        glBegin(GL_LINE_STRIP);
        for (int j=0;j<CONTOUR_RES;j++) {
            double y = tree_contour[i][j];
            double x = ( 2*j/(double)(CONTOUR_RES-1) - 1 ) * y/(double)max_display_height;
//          printf("Vertex x[%d] = %lf y=%lf \n", j,x,y);
            glVertex2f((float) (x/2.+0.5), (float) y);
        }
        glEnd();
    }

    //
    // Draw the current contour
    //
    double r = btree->nodes[0].sqrt_radius;
    
    if ( hier_info->database )
    {
        //SID: drawContours(r*cluster_radius,0.,tree_brush[0], 2.0);
        drawContours(r*hier_info->brushed_radius,hier_info->tree_brush[0], hier_info->tree_brush[1], 3.0);
        //SID: drawContours(r*cluster_radius,tree_brush[1], 1.0, 2.0);
    }
    else
    {
        drawContours(r*hier_info->cluster_radius,0.,hier_info->tree_brush[0], 2.0);
        drawContours(r*hier_info->brushed_radius,hier_info->tree_brush[0], hier_info->tree_brush[1], 3.0);
        drawContours(r*hier_info->cluster_radius,hier_info->tree_brush[1], 1.0, 2.0);
    }

    //
    // Draw the labels: tree depth, current depth
    //
    if (hier_info->tree_height == 0) FindTreeHeight(&btree->nodes[0],0,&(hier_info->tree_height));
    char str[30];
    sprintf(str,"Tree Depth:%d", hier_info->tree_height);
    canvas->SetForeground(0xffffff);
    canvas->DrawString(0,0.5,str);

}
*/

/*
void CHierNav::Draw()
{
    if (!dataset || !btree) return;

    canvas->MakeCurrent();

    //
    // Setup window parameters.
    //
    int W,H;
    canvas->GetSize(&W,&H);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glViewport( BORDER, BOT_BORDER, W-2*BORDER, H-BOT_BORDER-TOP_BORDER);
    glOrtho(-1,+1,max_display_height+.001,0,-1,1);

    canvas->SetForeground(0xaaaaaa);
    canvas->SetBackground(0x000000);
    canvas->Clear();

    //
    // Draw the brush and brush handles
    //
    glColor3f((float) .5, (float) .1, (float) .3);
/*
    static unsigned char *stipple = 0;
    if (stipple == 0) {
        stipple = new unsigned char[128];
        for (int i=0; i<128; i++) {
            stipple[i] = rand()%256;
        }
    }

    glPolygonStipple(stipple);
    if (EXTENT_BRUSH) glEnable(GL_POLYGON_STIPPLE);
*/
/*    glBegin(GL_POLYGON);
    glVertex2f(0, 0);
    glVertex2f((float) (2*hier_info->tree_brush[0]-1), (float) max_display_height);
    glVertex2f((float) (2*hier_info->tree_brush[1]-1), (float) max_display_height);
    glEnd();

//    glDisable(GL_POLYGON_STIPPLE);

    glColor3f((float) .8, (float) .8, (float) .8);
    glPointSize(5.0);
    glBegin(GL_POINTS);
    glVertex2f((float) (2*hier_info->tree_brush[0]-1), (float) max_display_height);
    glVertex2f((float) (2*hier_info->tree_brush[1]-1), (float) max_display_height);
    glEnd();
    glPointSize(1.0);


    //
    // Draw the outer triangle
    //
    glColor3f((float) .8, (float) .8, (float) 0);
    glBegin(GL_LINE_LOOP);
    glVertex2f(0, 0);
    glVertex2f((float) (+1), (float) max_display_height);
    glVertex2f((float) (-1), (float) max_display_height);
    glEnd();

    //
    // Draw the contours
    //
    glColor3f((float) .8, (float) .8, (float) .8);
    int i;
    for (i=0;i<tree_contour.Len();i++) {
i=tree_contour.Len()-1; // only draw the last contour
        glBegin(GL_LINE_STRIP);
        for (int j=0;j<CONTOUR_RES;j++) {
            double y = tree_contour[i][j];
            double x = ( 2*j/(double)(CONTOUR_RES-1) - 1 ) * y/(double)max_display_height;
//          printf("Vertex x[%d] = %lf y=%lf \n", j,x,y);
            glVertex2f((float) x, (float) y);
        }
        glEnd();
    }

    //
    // Draw the current contour
    //
    double r = btree->nodes[0].sqrt_radius;
    
    if ( hier_info->database )
    {
        //SID: drawContours(r*cluster_radius,0.,tree_brush[0], 2.0);
        drawContours(r*hier_info->brushed_radius,hier_info->tree_brush[0], hier_info->tree_brush[1], 3.0);
        //SID: drawContours(r*cluster_radius,tree_brush[1], 1.0, 2.0);
    }
    else
    {
        drawContours(r*hier_info->cluster_radius,0.,hier_info->tree_brush[0], 2.0);
        drawContours(r*hier_info->brushed_radius,hier_info->tree_brush[0], hier_info->tree_brush[1], 3.0);
        drawContours(r*hier_info->cluster_radius,hier_info->tree_brush[1], 1.0, 2.0);
    }

    //
    // Draw the labels: tree depth, current depth
    //
    if (hier_info->tree_height == 0) FindTreeHeight(&btree->nodes[0],0,&(hier_info->tree_height));
    char str[30];
    sprintf(str,"Tree Depth:%d", hier_info->tree_height);
    canvas->SetForeground(0xffffff);
    canvas->DrawString(-1,0.5,str);

    glViewport( BORDER, BORDER, W-2*BORDER, 2*BORDER);
    hier_info->CMAP.DrawColorBar();
}
*/

enum MOUSE_MODE {
    INACTIVE,
    LEFT_BRUSH_HANDLE,
    RIGHT_BRUSH_HANDLE,
    MIDDLE_BRUSH,
    INNER_CONTOUR,
    OUTER_CONTOUR
};

static MOUSE_MODE near_mouse_mode = INACTIVE;
static MOUSE_MODE mouse_mode = INACTIVE;

static int last_pick_x;

//extern HistMgmt *histMgmt;	//YK

/*void CHierNav::InputMouseMotion(int x, int y)
{
	if(histMgmt->already_started == 0)
	{
		histMgmt->already_started = 1;
		histMgmt->setInitialTime();
	}

	//YakovS**************************************************************************

    //FILE *newfptr = fopen(newfile,"w");

	FILE *histfptr = fopen("history.ses","a");
	

	 you would use the following to change the extention
    int N = strlen(newfile);
	char* xyFile = new char[N];
	strcpy(xyFile,newfile);
	strcpy(&xyFile[N-3],".xy");
	

	//(for reference)      fprintf(histfptr, "%d \n", dimension);

	//YakovE************************************************************************
    
	if (!dataset || !btree) return;

    int W,H;
    canvas->GetSize(&W,&H);

    //
    // Map mouse coordinates to triangular coordinates [0,1], [0,1]
    //
    double xx = (x-BORDER)/(double)(W-2*BORDER);
    double yy = 1.0 - (y-BOT_BORDER)/(double)(H-TOP_BORDER-BOT_BORDER);
    if (yy!=0) xx /= yy;

    //
    // Check if we are clicking on the brush handles
    //
    #define NEARHANDLES 0.05
    near_mouse_mode = INACTIVE;

    if (mouse_mode == INACTIVE) {

        double L = hier_info->tree_brush[0];
        double R = hier_info->tree_brush[1];

        if (fabs(yy-1)<NEARHANDLES) {
            if ( fabs(xx) < NEARHANDLES && fabs(L) < NEARHANDLES && fabs(R) < NEARHANDLES )
            {
                canvas->SetCursor("left_side");
                near_mouse_mode = RIGHT_BRUSH_HANDLE;
            }
            else if ( xx>L && xx<R ) {

                canvas->SetCursor("sb_h_double_arrow");
                near_mouse_mode = MIDDLE_BRUSH;
            } 
            else if ( fabs(xx-L) < NEARHANDLES ) {

                canvas->SetCursor("right_side");
                near_mouse_mode = LEFT_BRUSH_HANDLE;
            } 
            else if ( fabs(xx-R) < NEARHANDLES ) {
                canvas->SetCursor("left_side");
                near_mouse_mode = RIGHT_BRUSH_HANDLE;
            }
        } else {
            if ( xx<0 || xx>1 || yy<0 || yy>1) {

                canvas->SetCursor("cross");
            } else
            if ( xx>L && xx< R ) {
                near_mouse_mode = INNER_CONTOUR;
                canvas->SetCursor("sb_v_double_arrow");
            } else {
                near_mouse_mode = OUTER_CONTOUR;
                canvas->SetCursor("double_arrow");
            }
        }
    }

    if (canvas->button[0] != true) return;

    if ( hier_info->database )
    {
        switch(mouse_mode) {
        case LEFT_BRUSH_HANDLE:

		// SID 
	    save_active_set();

            hier_info->tree_brush[0] = xx;
            if (hier_info->tree_brush[0]<0) hier_info->tree_brush[0]=0;
            if (hier_info->tree_brush[0]>hier_info->tree_brush[1]) hier_info->tree_brush[0]=hier_info->tree_brush[1];

		//YS
		//fprintf(histfptr, "%lf\t", histMgmt->getInitialTime());
		fprintf(histfptr, "%d\t", (int)histMgmt->getAbsoluteTime());
		fprintf(histfptr, "%d\t%lf\n", A_HIER_L_B_H, hier_info->tree_brush[0]);
		//fprintf(histfptr, "LEFT_BRUSH_HANDLE: \t%lf\n", hier_info->tree_brush[0]);
		fclose(histfptr);
		//YE

	    // SID 
	    register_change();

            SetupBrushParameters();
            appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));


            return;

        case RIGHT_BRUSH_HANDLE:

		// SID 
	    save_active_set();

            hier_info->tree_brush[1] = xx;
            if (hier_info->tree_brush[1]>1) hier_info->tree_brush[1]=1;
            if (hier_info->tree_brush[1]<hier_info->tree_brush[0]) hier_info->tree_brush[1]=hier_info->tree_brush[0];

	    //YS
		fprintf(histfptr, "%d\t", (int)histMgmt->getAbsoluteTime());
		fprintf(histfptr, "%d\t%lf\n", A_HIER_R_B_H, hier_info->tree_brush[1]);
		//fprintf(histfptr, "RIGHT_BRUSH_HANDLE: \t%lf\n", hier_info->tree_brush[1]);
		fclose(histfptr);
		//YE

	    // SID 
	    register_change();

            SetupBrushParameters();
            appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
            return;

        case MIDDLE_BRUSH: {
			
		// SID 
	    save_active_set();

            double prev_x = (last_pick_x-BORDER)/(double)(W-2*BORDER);
            if (yy!=0) prev_x /= yy;

            double delta_x = xx - prev_x;

            hier_info->tree_brush[0] += delta_x;
            hier_info->tree_brush[1] += delta_x;

            if (hier_info->tree_brush[1]>1) {
                delta_x = hier_info->tree_brush[1]-1;
                hier_info->tree_brush[0] -= delta_x;
                hier_info->tree_brush[1] -= delta_x;
            } else
            if (hier_info->tree_brush[0]<0) {
                delta_x = hier_info->tree_brush[0];
                hier_info->tree_brush[0] -= delta_x;
                hier_info->tree_brush[1] -= delta_x;
            }

            last_pick_x = x;

	    //YS
		fprintf(histfptr, "%d\t", (int)histMgmt->getAbsoluteTime());
		fprintf(histfptr, "%d\t%lf\n", A_HIER_M_B, delta_x);
		//fprintf(histfptr, "MIDDLE_BRUSH: \t%lf\n", delta_x);
			fclose(histfptr);
		//YE


	    // SID 
	    register_change();

            SetupBrushParameters();
            appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
            return;
            }

        default:
            break;
        }
    }
    else
    {
        switch(mouse_mode) {
        case LEFT_BRUSH_HANDLE:
            hier_info->tree_brush[0] = xx;
            if (hier_info->tree_brush[0]<0) hier_info->tree_brush[0]=0;
            if (hier_info->tree_brush[0]>hier_info->tree_brush[1]) hier_info->tree_brush[0]=hier_info->tree_brush[1];

            SetupBrushParameters();
            appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));

            return;

        case RIGHT_BRUSH_HANDLE:
            hier_info->tree_brush[1] = xx;
            if (hier_info->tree_brush[1]>1) hier_info->tree_brush[1]=1;
            if (hier_info->tree_brush[1]<hier_info->tree_brush[0]) hier_info->tree_brush[1]=hier_info->tree_brush[0];

            SetupBrushParameters();
            appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
            return;

        case MIDDLE_BRUSH: {

            double prev_x = (last_pick_x-BORDER)/(double)(W-2*BORDER); 
            if (yy!=0) prev_x /= yy;

            double delta_x = xx - prev_x;

            hier_info->tree_brush[0] += delta_x;
            hier_info->tree_brush[1] += delta_x;

            if (hier_info->tree_brush[1]>1) {
                delta_x = hier_info->tree_brush[1]-1;
                hier_info->tree_brush[0] -= delta_x;
                hier_info->tree_brush[1] -= delta_x;
            } else
            if (hier_info->tree_brush[0]<0) {
                delta_x = hier_info->tree_brush[0];
                hier_info->tree_brush[0] -= delta_x;
                hier_info->tree_brush[1] -= delta_x;
            }

            last_pick_x = x;

            SetupBrushParameters();
            appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
            return;
            }

        default:
            break;
        }
    }
    //
    // Reject if picking outside triangle
    //
    if (xx<0 || xx>1 || yy<0 || yy>1) return;

    //
    // Map to contour coordinates [0,CONTOUR_RES], [0,max_display_height]
    //
    yy *= max_display_height;

    xx = xx * (CONTOUR_RES-1);

    int ixx = (int) xx;
    double ratio = xx-ixx;

    //
    // Find out which contour bounds "yy" at position "xx"
    //
    double cy = 0, prev_cy;
    int i;
    for (i=0;i<tree_contour.Len();i++) {
        prev_cy = cy;
        cy = tree_contour[i][ixx];
        if (ixx<CONTOUR_RES-1) cy += ratio*(tree_contour[i][ixx+1] - tree_contour[i][ixx]);
        if (cy > yy) break;
    }

    double radius;
    if (i==tree_contour.Len()) {
        radius = 0;
    } else if (i==0){
        radius = btree->nodes[0].sqrt_radius;
    } else{
        ratio = (yy-prev_cy)/(cy-prev_cy);
        radius = (1-ratio)*contour_radius[i-1] + (ratio)*contour_radius[i];
    }

    double root_radius = btree->nodes[0].sqrt_radius;
    radius /= root_radius;

    if ( hier_info->database )
    {
        switch(mouse_mode)
        {
            case INNER_CONTOUR:
				//YS
				fprintf(histfptr, "%d\t", (int)histMgmt->getAbsoluteTime());
				fprintf(histfptr, "%d\t%lf\n", A_HIER_I_C, radius);
				//fprintf(histfptr, "INNER_CONTOUR: \t%lf\n", radius);
				fclose(histfptr);
				//YE
				
				// SID 
	        save_active_set();

                hier_info->brushed_radius = radius;
                hierMarkContourNodes( root_radius*hier_info->brushed_radius, IS_CONTOUR1 );

                // SID 
                register_change();

                appUpdateClusterRadiusSliders(Togl_Interp(canvas->GetWidget()));
                break;

            case OUTER_CONTOUR:
                //YS
				fprintf(histfptr, "%d\t", (int)histMgmt->getAbsoluteTime());
				fprintf(histfptr, "%d\t%lf\n", A_HIER_O_C, radius);
				//fprintf(histfptr, "OUTER_CONTOUR: \t%lf\n", radius);
				fclose(histfptr);
				//YE
				
				// SID 
	        save_active_set();

                hier_info->cluster_radius = radius;
                hierMarkContourNodes( root_radius*hier_info->cluster_radius, IS_CONTOUR0 );

                // SID
                register_change();

                appUpdateClusterRadiusSliders(Togl_Interp(canvas->GetWidget()));
                break;

            default:
                break;
        }
    }
    else
    {
        switch(mouse_mode)
        {
            case INNER_CONTOUR:
                hier_info->brushed_radius = radius;
                hierMarkContourNodes( root_radius*hier_info->brushed_radius, IS_CONTOUR1 );
                appUpdateClusterRadiusSliders(Togl_Interp(canvas->GetWidget()));
                break;

            case OUTER_CONTOUR:
                hier_info->cluster_radius = radius;
                hierMarkContourNodes( root_radius*hier_info->cluster_radius, IS_CONTOUR0 );
                appUpdateClusterRadiusSliders(Togl_Interp(canvas->GetWidget()));
                break;
            default:
                break;
        }
    }

    appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
}

*/

void CHierNav::ShowCaption(const char* message)
{ 
    appShowCanvasMessage(Togl_Interp(this->canvas->GetWidget()), message, this->canvas->name);
}

void CHierNav::InputMouseMotion(int x, int y)
{
	if(histMgmt->already_started == 0)
	{
		histMgmt->already_started = 1;
		histMgmt->setInitialTime();
	}

	//YakovS**************************************************************************

    //FILE *newfptr = fopen(newfile,"w");

//	FILE *histfptr = fopen("history.ses","a");
	

	/* you would use the following to change the extention
    int N = strlen(newfile);
	char* xyFile = new char[N];
	strcpy(xyFile,newfile);
	strcpy(&xyFile[N-3],".xy");
	*/

	//(for reference)      fprintf(histfptr, "%d \n", dimension);

	//YakovE************************************************************************
    
	if (!dataset || !btree) return;

    Vec2 p = canvas->MapScreenToData( Vec2(x,y) );
    
    //
    // Map mouse coordinates to triangular coordinates [-1,1], [0,1]
    //
//    double xx = 2*(p.X-BORDER)/(double)(W-2*BORDER) - 1;
//    double yy = 1.0 - (p.Y-BOT_BORDER)/(double)(H-TOP_BORDER-BOT_BORDER);
    double xx = 2 * p.X - 1;
    double yy = p.Y /(1-BARHIGHT);
    if (yy!=0) xx /= yy;

    //
    // Check if we are clicking on the brush handles
    //
    #define NEARHANDLES 0.05
    near_mouse_mode = INACTIVE;

    if (mouse_mode == INACTIVE) {

        double L = 2*hier_info->tree_brush[0]-1;
        double R = 2*hier_info->tree_brush[1]-1;

        if (fabs(yy-1)<NEARHANDLES) {
            if ( fabs(xx+1) < NEARHANDLES && fabs(L+1) < NEARHANDLES && fabs(R+1) < NEARHANDLES )
            {
                canvas->SetCursor("left_side");
                ShowCaption("Move right brush handle");
                near_mouse_mode = RIGHT_BRUSH_HANDLE;
            }
            else if ( xx>L && xx<R ) {

                canvas->SetCursor("sb_h_double_arrow");
                ShowCaption("Shift brushed region");
                near_mouse_mode = MIDDLE_BRUSH;
            } 
            else if ( fabs(xx-L) < NEARHANDLES ) {

                canvas->SetCursor("right_side");
                ShowCaption("Move left brush handle");
                near_mouse_mode = LEFT_BRUSH_HANDLE;
            } 
            else if ( fabs(xx-R) < NEARHANDLES ) {
                canvas->SetCursor("left_side");
                ShowCaption("Move right brush handle");
                near_mouse_mode = RIGHT_BRUSH_HANDLE;
            }
        } else {
            if ( xx<-1 || xx>1 || yy<0 || yy>1) {
                ShowCaption("Inactive area");
                canvas->SetCursor("cross");
            } else
            if ( xx>L && xx< R ) {
                near_mouse_mode = INNER_CONTOUR;
                ShowCaption("Change brushed cluster radius");
                canvas->SetCursor("sb_v_double_arrow");
            } else {
                near_mouse_mode = OUTER_CONTOUR;
                ShowCaption("Change non-brushed cluster radius");
                canvas->SetCursor("double_arrow");
            }
        }
    }

    if (canvas->button[0] != true) return;

    if ( hier_info->database )
    {
        switch(mouse_mode) {
        case LEFT_BRUSH_HANDLE:

		/* SID */
	    save_active_set();

            hier_info->tree_brush[0] = (xx+1)*.5;
            if (hier_info->tree_brush[0]<0) hier_info->tree_brush[0]=0;
            if (hier_info->tree_brush[0]>hier_info->tree_brush[1]) hier_info->tree_brush[0]=hier_info->tree_brush[1];

		//YS
		//fprintf(histfptr, "%lf\t", histMgmt->getInitialTime());
//		fprintf(histfptr, "%d\t", (int)histMgmt->getAbsoluteTime());
//		fprintf(histfptr, "%d\t%lf\n", A_HIER_L_B_H, hier_info->tree_brush[0]);
		//fprintf(histfptr, "LEFT_BRUSH_HANDLE: \t%lf\n", hier_info->tree_brush[0]);
//		fclose(histfptr);
		//YE

	    /* SID */
	    register_change();

            appUpdateTreeBrushEntries(Togl_Interp(canvas->GetWidget()));
            SetupBrushParameters();
            appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));


            return;

        case RIGHT_BRUSH_HANDLE:

		/* SID */
	    save_active_set();

            hier_info->tree_brush[1] = (xx+1)*.5;
            if (hier_info->tree_brush[1]>1) hier_info->tree_brush[1]=1;
            if (hier_info->tree_brush[1]<hier_info->tree_brush[0]) hier_info->tree_brush[1]=hier_info->tree_brush[0];

	    //YS
//		fprintf(histfptr, "%d\t", (int)histMgmt->getAbsoluteTime());
//		fprintf(histfptr, "%d\t%lf\n", A_HIER_R_B_H, hier_info->tree_brush[1]);
		//fprintf(histfptr, "RIGHT_BRUSH_HANDLE: \t%lf\n", hier_info->tree_brush[1]);
//		fclose(histfptr);
		//YE

	    /* SID */
	    register_change();

            appUpdateTreeBrushEntries(Togl_Interp(canvas->GetWidget()));
            SetupBrushParameters();
            appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
            return;

        case MIDDLE_BRUSH: {
			
		/* SID */
	    save_active_set();

            Vec2 p1 = canvas->MapScreenToData( Vec2(last_pick_x,0.5) );
            double prev_x = 2*p1.X - 1;
            if (yy!=0) prev_x /= yy;

            double delta_x = (xx - prev_x)/2.0;

            hier_info->tree_brush[0] += delta_x;
            hier_info->tree_brush[1] += delta_x;

            if (hier_info->tree_brush[1]>1) {
                delta_x = hier_info->tree_brush[1]-1;
                hier_info->tree_brush[0] -= delta_x;
                hier_info->tree_brush[1] -= delta_x;
            } else
            if (hier_info->tree_brush[0]<0) {
                delta_x = hier_info->tree_brush[0];
                hier_info->tree_brush[0] -= delta_x;
                hier_info->tree_brush[1] -= delta_x;
            }

            last_pick_x = x;

	    //YS
//		fprintf(histfptr, "%d\t", (int)histMgmt->getAbsoluteTime());
//		fprintf(histfptr, "%d\t%lf\n", A_HIER_M_B, delta_x);
		//fprintf(histfptr, "MIDDLE_BRUSH: \t%lf\n", delta_x);
//			fclose(histfptr);
		//YE


	    /* SID */
	    register_change();

            appUpdateTreeBrushEntries(Togl_Interp(canvas->GetWidget()));
            SetupBrushParameters();
            appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
            return;
            }

        default:
            break;
        }
    }
    else
    {
        switch(mouse_mode) {
        case LEFT_BRUSH_HANDLE:
            hier_info->tree_brush[0] = (xx+1)*.5;
            if (hier_info->tree_brush[0]<0) hier_info->tree_brush[0]=0;
            if (hier_info->tree_brush[0]>hier_info->tree_brush[1]) hier_info->tree_brush[0]=hier_info->tree_brush[1];

            appUpdateTreeBrushEntries(Togl_Interp(canvas->GetWidget()));
            SetupBrushParameters();
            appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));

            return;

        case RIGHT_BRUSH_HANDLE:
            hier_info->tree_brush[1] = (xx+1)*.5;
            if (hier_info->tree_brush[1]>1) hier_info->tree_brush[1]=1;
            if (hier_info->tree_brush[1]<hier_info->tree_brush[0]) hier_info->tree_brush[1]=hier_info->tree_brush[0];

            appUpdateTreeBrushEntries(Togl_Interp(canvas->GetWidget()));
            SetupBrushParameters();
            appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
            return;

        case MIDDLE_BRUSH: {

            Vec2 p1 = canvas->MapScreenToData( Vec2(last_pick_x,0.5) );
            double prev_x = 2*p1.X - 1;

//            double prev_x = 2*(last_pick_x-BORDER)/(double)(W-2*BORDER) - 1;
            if (yy!=0) prev_x /= yy;

            double delta_x = (xx - prev_x)/2.0;

            hier_info->tree_brush[0] += delta_x;
            hier_info->tree_brush[1] += delta_x;

            if (hier_info->tree_brush[1]>1) {
                delta_x = hier_info->tree_brush[1]-1;
                hier_info->tree_brush[0] -= delta_x;
                hier_info->tree_brush[1] -= delta_x;
            } else
            if (hier_info->tree_brush[0]<0) {
                delta_x = hier_info->tree_brush[0];
                hier_info->tree_brush[0] -= delta_x;
                hier_info->tree_brush[1] -= delta_x;
            }

            last_pick_x = x;

            appUpdateTreeBrushEntries(Togl_Interp(canvas->GetWidget()));
            SetupBrushParameters();
            appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
            return;
            }

        default:
            break;
        }
    }
    //
    // Reject if picking outside triangle
    //
    if (xx<-1 || xx>1 || yy<0 || yy>1) return;

    //
    // Map to contour coordinates [0,CONTOUR_RES], [0,max_display_height]
    //
    yy *= max_display_height;

    xx = ((xx + 1)/2.0) * (CONTOUR_RES-1);

    int ixx = (int) xx;
    double ratio = xx-ixx;

    //
    // Find out which contour bounds "yy" at position "xx"
    //
    double cy = 0, prev_cy;
    int i;
    for (i=0;i<tree_contour.Len();i++) {
        prev_cy = cy;
        cy = tree_contour[i][ixx];
        if (ixx<CONTOUR_RES-1) cy += ratio*(tree_contour[i][ixx+1] - tree_contour[i][ixx]);
        if (cy > yy) break;
    }

    double radius;
    if (i==tree_contour.Len()) {
        radius = 0;
    } else if (i==0){
        radius = btree->nodes[0].sqrt_radius;
    } else{
        ratio = (yy-prev_cy)/(cy-prev_cy);
        radius = (1-ratio)*contour_radius[i-1] + (ratio)*contour_radius[i];
    }

    double root_radius = btree->nodes[0].sqrt_radius;
    radius /= root_radius;

    if ( hier_info->database )
    {
        switch(mouse_mode)
        {
            case INNER_CONTOUR:
				//YS
//				fprintf(histfptr, "%d\t", (int)histMgmt->getAbsoluteTime());
//				fprintf(histfptr, "%d\t%lf\n", A_HIER_I_C, radius);
				//fprintf(histfptr, "INNER_CONTOUR: \t%lf\n", radius);
//				fclose(histfptr);
				//YE
				
				/* SID */
	        save_active_set();

                hier_info->brushed_radius = radius;
                hierMarkContourNodes( root_radius*hier_info->brushed_radius, IS_CONTOUR1 );

                /* SID */
                register_change();

                appUpdateClusterRadiusSliders(Togl_Interp(canvas->GetWidget()));
                break;

            case OUTER_CONTOUR:
                //YS
//				fprintf(histfptr, "%d\t", (int)histMgmt->getAbsoluteTime());
//				fprintf(histfptr, "%d\t%lf\n", A_HIER_O_C, radius);
				//fprintf(histfptr, "OUTER_CONTOUR: \t%lf\n", radius);
//				fclose(histfptr);
				//YE
				
				/* SID */
	        save_active_set();

                hier_info->cluster_radius = radius;
                hierMarkContourNodes( root_radius*hier_info->cluster_radius, IS_CONTOUR0 );

                /* SID */
                register_change();

                appUpdateClusterRadiusSliders(Togl_Interp(canvas->GetWidget()));
                break;

            default:
                break;
        }
    }
    else
    {
        switch(mouse_mode)
        {
            case INNER_CONTOUR:
                hier_info->brushed_radius = radius;
                hierMarkContourNodes( root_radius*hier_info->brushed_radius, IS_CONTOUR1 );
                appUpdateClusterRadiusSliders(Togl_Interp(canvas->GetWidget()));
                break;

            case OUTER_CONTOUR:
                hier_info->cluster_radius = radius;
                hierMarkContourNodes( root_radius*hier_info->cluster_radius, IS_CONTOUR0 );
                appUpdateClusterRadiusSliders(Togl_Interp(canvas->GetWidget()));
                break;
            default:
                break;
        }
    }

    appRefreshAllCanvas(Togl_Interp(canvas->GetWidget()));
}


static int prev_normal_style, prev_brush_style;

void CHierNav::InputButtonPress(int button, int x, int y)
{
    if (near_mouse_mode!=INACTIVE) mouse_mode = near_mouse_mode;
    last_pick_x = x;

    prev_normal_style = hier_info->normal_style;
    prev_brush_style = hier_info->highlight_style;

//??    normal_style |= DIM;

    InputMouseMotion(x, y);
}

void CHierNav::InputButtonRelease(int button, int x, int y)
{
    hier_info->normal_style = prev_normal_style;
    hier_info->highlight_style = prev_brush_style;

    near_mouse_mode = mouse_mode = INACTIVE;
    InputMouseMotion(x, y);
}
